//=============================================================================
// Copyright (c) 2016-present Allan CORNET (Nelson)
//=============================================================================
// This file is part of the Nelson.
//=============================================================================
// LICENCE_BLOCK_BEGIN
// SPDX-License-Identifier: LGPL-3.0-or-later
// LICENCE_BLOCK_END
//=============================================================================
#include <boost/format.hpp>
#include "NargIn.hpp"
#include "Error.hpp"
#include "MacroFunctionDef.hpp"
#include "characters_encoding.hpp"
//=============================================================================
namespace Nelson {
//=============================================================================
int
NargIn(Evaluator* eval, const std::wstring& functionName)
{
    FunctionDefPtr fptr = nullptr;
    bool bIsFun = eval->lookupFunction(wstring_to_utf8(functionName), fptr);
    if (bIsFun) {
        if (fptr->type() == NLS_MACRO_FUNCTION) {
            return ((MacroFunctionDef*)(fptr))->nargin();
        }
        if (fptr->type() == NLS_MEX_FUNCTION) {
            std::string msg
                = str(boost::format(_("'%s' does not know how to answer nargin/nargout."))
                    % fptr->getName());
            Error(msg);
        }
        return fptr->inputArgCount();
    }
    Error(_W("function not found."));
    return -1;
}
//=============================================================================
} // namespace Nelson
//=============================================================================
